import math
import cv2 as cv
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.axes import Axes


def show(*inputs: cv.Mat | tuple[str, cv.Mat], ncols=3, height=5, width=5, axis=False, cmap="gray") -> Figure:
    """Shows a set of images or (title, image) pairs"""
    def show_input(ax: Axes, input: cv.Mat | tuple[str, cv.Mat]):
        match input:
            case title, img:
                ax.imshow(img, cmap=cmap)
                ax.set_title(title)
            case img:
                ax.imshow(img, cmap=cmap)
        ax.axis(axis)
        
    n = len(inputs)
    if n == 0:
        return None
    if n == 1:
        fig, ax = plt.subplots(figsize=(width, height))
        show_input(ax, inputs[0])
    elif n <= ncols:
        fig, ax = plt.subplots(1, n, figsize=(5*n, height))
        for i, input in enumerate(inputs):
            show_input(ax[i], input)
    elif ncols == 1:
        nrows = n
        fig, ax = plt.subplots(nrows, 1, figsize=(5, height*nrows))
        for i, input in enumerate(inputs):
            show_input(ax[i], input)
    else:
        nrows = math.ceil(n / ncols)
        fig, ax = plt.subplots(nrows, ncols, figsize=(5*ncols, height*nrows))
        for i, input in enumerate(inputs):
            show_input(ax[i // ncols, i % ncols], input)
    
    plt.close(fig)
    return fig


from .opencv import imread, grayscale, threshold, dilate, erode, cvtColor, imdecode
try:
  from .plot import show
except ImportError:
  ...
from .encoding import b64encode, b64decode, encode, decode
from .colors import mod_color
from .misc import black
from .rescaling import descale_h, rescale_h
from . import draw



import cv2 as cv
import numpy as np
import ramda as R

N = int; _1 = int; _2 = int; _4 = int

Img = cv.Mat
Img3D = cv.Mat

@R.curry
def vertices(
    vertices: np.ndarray,
    img: Img,
    radius: int = 10,
    color: int | tuple[int] = (255, 0, 0)
):
    if isinstance(color, tuple):
        img = cv.cvtColor(img, cv.COLOR_GRAY2BGR) if len(np.shape(img)) == 2 else np.copy(img)
        
    output = img.copy()
    for [x, y] in vertices:
        cv.circle(output, (int(x), int(y)), radius=radius, color=color, thickness=-1)
        
    return output

@R.curry
def vlines(xs: list[float], img: Img, color = (255, 0, 0), thickness = 3):
    height = img.shape[0]
    ls = np.array([[[x, 0, x, height-1]] for x in xs], dtype=np.int32)
    return lines(ls, img, color, thickness)

@R.curry
def hlines(ys: list[float], img: Img, color = (255, 0, 0), thickness = 3):
    width = img.shape[1]
    ls = np.array([[[0, y, width-1, y]] for y in ys], dtype=np.int32)
    return lines(ls, img, color, thickness)

@R.curry
def lines(
    lines: tuple[N, _1, _4],
    img: Img,
    color = (255, 0, 0),
    thickness = 3
):
    out = img.copy()
    for line in lines:
        x1, y1, x2, y2 = line[0]
        cv.line(out, [x1, y1], [x2, y2], color=color, thickness=thickness)
        
    return out

@R.curry
def contours(
    contours: list[np.ndarray[tuple[N, _1, _2]]],
    img: Img,
    color: int | tuple[int] = (255, 0, 0),
    thickness = 3
) -> Img:
    if isinstance(color, tuple):
        outimg = cv.cvtColor(img, cv.COLOR_GRAY2BGR) if len(np.shape(img)) == 2 else np.copy(img)
    else:
        outimg = np.copy(img) if len(np.shape(img)) == 2 else cv.cvtColor(img, cv.COLOR_BGR2GRAY)
        
    return cv.drawContours(outimg, [np.array(c, dtype=np.int32) for c in contours], -1, color, thickness)


import cv2 as cv
import ramda as R

@R.curry
def rescale_h(img: cv.Mat, height: int) -> cv.Mat:
    """Rescales the image to `height`"""
    h, w = img.shape[:2]
    width = int(height * w/h)
    return cv.resize(img, (width, height))

@R.curry
def descale_h(img: cv.Mat, target_height: int) -> cv.Mat:
    """Rescales the image to `height = min(img.height, target_height)`"""
    cur_height, _ = img.shape[:2]
    return rescale_h(img, target_height) if target_height < cur_height else img
